import asyncio
import re
from pyplanet.apps.config import AppConfig
from pyplanet.apps.core.maniaplanet import callbacks as mp_signals
from pyplanet.contrib.setting import Setting
from .views import ChatView, ChatUpdaterView


class emotes(AppConfig):
	app_dependencies = ['core.maniaplanet']

	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		self.player_chats = dict()

		self.chatlines = []
		self.chatlinesPrev = []
		self.context.signals.listen(mp_signals.player.player_connect, self.player_connect)
		self.context.signals.listen(mp_signals.player.player_disconnect, self.player_disconnect)
		self.context.signals.listen(mp_signals.player.player_chat, self.player_chat)

		self.chatwidget = ChatView(self)
		self.chatupdater = ChatUpdaterView(self)
		# self.random_future = asyncio.ensure_future(self.messages_loop())
		# Settings
		self.setting_emote_data = Setting(
			'emoji_data', 'JsonData for emojis plugin', Setting.CAT_OTHER, type=str,
			description='JsonData for emojis plugin', change_target=self.reload_settings,
			default="""
			{"bitmap":[16,1],"emojis":[{"name":"heart","offset":[0,0]},{"name":"3Head","offset":[1,0]},{"name":"4Head","offset":[2,0]},{"name":"4Weird","offset":[3,0]},{"name":"5Head","offset":[4,0]},{"name":"CurseLit","offset":[5,0]},{"name":"FeelsOkayMan","offset":[6,0]},{"name":"HeyGuys","offset":[7,0]},{"name":"Kapp","offset":[8,0]},{"name":"kappa","offset":[9,0]},{"name":"Keepo","offset":[10,0]},{"name":"KEKW","offset":[11,0]},{"name":"Lul","offset":[12,0]},{"name":"LULW","offset":[13,0]},{"name":"monkaW","offset":[14,0]},{"name":"moon2E","offset":[15,0]},{"name":"NotLikeThis","offset":[0,1]},{"name":"OMEGALUL","offset":[1,1]},{"name":"PepeLaugh","offset":[2,1]},{"name":"PJSalt","offset":[3,1]},{"name":"POG","offset":[4,1]},{"name":"POGGERS","offset":[5,1]},{"name":"PogO","offset":[6,1]},{"name":"PogU","offset":[7,1]},{"name":"Sadge","offset":[8,1]},{"name":"TriHard","offset":[9,1]},{"name":"WeirdChamp","offset":[10,1]},{"name":"YEP","offset":[11,1]}]}
			"""
		)
		self.setting_emote_url = Setting(
			'emoji_url', 'Emojis plugin Bitmap URL', Setting.CAT_OTHER, type=str,
			description='Emojis plugin Bitmap.', change_target=self.reload_settings,
			default="http://reaby.kapsi.fi/trackmania/emotes/emotes.png"
		)

	async def player_connect(self, player, is_spectator, source, signal):
		self.player_chats[player.login] = []
		await self.chatwidget.display(player_logins=[player.login])
		await self.chatupdater.display(player_logins=[player.login])

	async def player_disconnect(self,  player, reason, source, signal):
		self.player_chats.pop(player.login, None)

	async def on_start(self):
		for login in self.instance.player_manager.online_logins:
			self.player_chats[login] = []

		await self.context.setting.register(
			self.setting_emote_data,
			self.setting_emote_url,
		)
		self.instance.signals.listen(signal="pyplanet:chat", target=self.controller_message)

		await self.chatwidget.display()
		await self.instance.gbx.execute("ChatEnableManualRouting", True, True)

	# self.instance.ui_manager.properties.set_visibility('Race_Checkpoint', True)
	# await self.instance.ui_manager.properties.send_properties()

	async def reload_settings(self, *args, **kwargs):
		await self.chatwidget.display()

	async def controller_message(self, signal, text, logins):
		if logins is None:
			for login in self.instance.player_manager.online_logins:
				pla = self.player_chats[login]
				pla.append(text)
				if len(pla) > 30:
					del pla[0]
		else:
			for login in logins:
				pla = self.player_chats[login]
				pla.append(text)
				if len(pla) > 30:
					del pla[0]

		await self.chatupdater.display()

	async def player_chat(self, player, text, cmd):
		if "/" in text[0]:
			return

		for login in self.instance.player_manager.online_logins:
			pla = self.player_chats[login]
			pla.append(f"[{player.nickname}] {text}")
			if len(pla) > 30:
				del pla[0]

		await self.chatupdater.display()
